@@ -19,6 +19,15 @@
 namespace tensorrt_llm::kernels::cutlass_kernels {
 
 // Note update moe.py to match
-enum class ActivationType { Gelu = 0, Relu, Silu, Swiglu, Geglu, Identity, InvalidType };
+enum class ActivationType {
+  Gelu = 0,
+  Relu,
+  Silu,
+  Swiglu,
+  Geglu,
+  SwigluBias,
+  Identity,
+  InvalidType
+};
 
 }  // namespace tensorrt_llm::kernels::cutlass_kernels
