@@ -388,6 +388,28 @@ def get_batch_prefill_uri(
     )
 
 
+def get_batch_prefill_attention_sink_uri(
+    backend: str,
+    dtype_q: torch.dtype,
+    dtype_kv: torch.dtype,
+    dtype_o: torch.dtype,
+    dtype_idx: torch.dtype,
+    head_dim_qk: int,
+    head_dim_vo: int,
+    pos_encoding_mode: int,
+    use_sliding_window: bool,
+) -> str:
+    return (
+        f"batch_prefill_with_attention_sink_kv_cache_dtype_q_{filename_safe_dtype_map[dtype_q]}_"
+        f"dtype_kv_{filename_safe_dtype_map[dtype_kv]}_"
+        f"dtype_o_{filename_safe_dtype_map[dtype_o]}_"
+        f"dtype_idx_{filename_safe_dtype_map[dtype_idx]}_"
+        f"head_dim_qk_{head_dim_qk}_"
+        f"head_dim_vo_{head_dim_vo}_"
+        f"use_swa_{use_sliding_window}_" + ("_sm90" if backend == "fa3" else "")
+    )
+
+
 def get_batch_attention_uri(
     dtype_q: torch.dtype,
     dtype_kv: torch.dtype,
@@ -856,6 +878,54 @@ def gen_batch_prefill_module(
     )
 
 
+def gen_batch_prefill_attention_sink_module(
+    backend: str,
+    dtype_q: torch.dtype,
+    dtype_kv: torch.dtype,
+    dtype_o: torch.dtype,
+    dtype_idx: torch.dtype,
+    head_dim_qk: int,
+    head_dim_vo: int,
+    pos_encoding_mode: int,
+    use_sliding_window: bool,
+) -> JitSpec:
+    from flashinfer.jit.attention.variants import attention_sink_decl
+
+    uri = get_batch_prefill_attention_sink_uri(
+        backend,
+        dtype_q,
+        dtype_kv,
+        dtype_o,
+        dtype_idx,
+        head_dim_qk,
+        head_dim_vo,
+        pos_encoding_mode,
+        use_sliding_window,
+    )
+
+    return gen_customize_batch_prefill_module(
+        backend,
+        uri,
+        dtype_q,
+        dtype_kv,
+        dtype_o,
+        dtype_idx,
+        head_dim_qk,
+        head_dim_vo,
+        ["sink"],
+        ["float"],
+        ["sm_scale"],
+        ["double"],
+        "AttentionSink",
+        attention_sink_decl[backend],
+        pos_encoding_mode=pos_encoding_mode,
+        use_sliding_window=use_sliding_window,
+        use_logits_soft_cap=False,
+        use_fp16_qk_reduction=False,
+        fp8_enabled=False,
+    )
+
+
 def gen_batch_attention_module(
     dtype_q: torch.dtype,
     dtype_kv: torch.dtype,
@@ -1491,7 +1561,6 @@ def trtllm_gen_fmha_module():
     return gen_jit_spec(
         "fmha_gen",
         [
-            jit_env.FLASHINFER_CSRC_DIR / "trtllm_fmha_runner.cu",
             jit_env.FLASHINFER_CSRC_DIR / "trtllm_fmha_kernel_launcher.cu",
         ],
         extra_ldflags=["-lcuda"],
