@@ -283,16 +283,16 @@ struct BatchedGemmData {
     //
     float const* mPtrClampLimit{nullptr};
 
-    // The alpha and beta for SwiGlu.
+    // The alpha and beta for SwiGlu or GeGlu.
     // gatedActivation <- (x0 + beta) * activation(x1, alpha)
     // Shape is [B].
     // Alpha is 1.f if nullptr.
     // Beta is 0.f if nullptr.
-    // The formula:
+    // The formula for SwiGlu (for GeGlu, replace sigmoid with phi):
     //
-    //   out_glu  = x_glu * torch.sigmoid(alpha * x_glu) + (x_linear + beta)
-    float const* mPtrSwiGluAlpha{nullptr};
-    float const* mPtrSwiGluBeta{nullptr};
+    //   out_glu  = x_glu * torch.sigmoid(alpha * x_glu) * (x_linear + beta)
+    float const* mPtrGatedActAlpha{nullptr};
+    float const* mPtrGatedActBeta{nullptr};
 
     // Param is used when the kernel is configured with -routeAct true.
     // The inputs are not padded, but the outputs are padded to divUpMul(M[bi], tileM) for batchM or
@@ -460,8 +460,44 @@ class BatchedGemmInterface {
   // Returns the number of available cubin configurations
   size_t getNumBatchedGemmConfigs() const;
 
-  // Returns the number of CTAs of the last launched kernel.
-  int32_t getNumCtas() const { return mNumCtas; }
+  // Returns the grid dimensions of the current kernel.
+  std::tuple<int32_t, int32_t, int32_t> getGridDim(
+      BatchedGemmOptions const& options,
+      std::optional<int32_t> maxNumCtasInBatchDim = std::nullopt) const {
+    bool const batchM = options.mBatchMode == BatchedGemmOptions::BatchMode::BatchM;
+
+    int32_t numCtasBatch{0};
+    // For normal BMM, mNumTokens == 0 and the number of CTAs is known to host.
+    if (options.mIsStaticBatch) {
+      for (int32_t bi = 0; bi < options.mNumBatches; ++bi) {
+        numCtasBatch += batchM ? gemm::divUp(options.mBatchedM[bi], options.mTileM)
+                               : gemm::divUp(options.mBatchedN[bi], options.mTileN);
+      }
+    }
+    // For MoE, mNumTokens != 0 and the number of CTAs is known only at runtime.
+    // We launch maximally possible number of CTAs and use ptrNumNonExitingCtas to determine the
+    // actual number of CTAs to run.
+    else if ((options.mEnablesEarlyExit || options.mEnablesDelayedEarlyExit) &&
+             options.mNumTokens != 0) {
+      assert(maxNumCtasInBatchDim.has_value() &&
+             "maxNumCtasInBatchDim must be provided when options.mNumTokens != 0");
+      numCtasBatch = maxNumCtasInBatchDim.value();
+    } else {
+      throw std::invalid_argument("Invalid combination of options");
+    }
+
+    int32_t const numCtasTile =
+        batchM ? gemm::divUp(options.mN, options.mTileN) : gemm::divUp(options.mM, options.mTileM);
+    int32_t const numCtasInner = options.mNumSlicesForSplitK;
+    return std::make_tuple(numCtasBatch, numCtasTile, numCtasInner);
+  }
+
+  // Returns the number of CTAs of the current kernel.
+  int32_t getNumCtas(BatchedGemmOptions const& options,
+                     std::optional<int32_t> maxNumCtasInBatchDim = std::nullopt) const {
+    auto [numCtasBatch, numCtasTile, numCtasInner] = getGridDim(options, maxNumCtasInBatchDim);
+    return numCtasBatch * numCtasTile * numCtasInner;
+  }
 
   // Returns true if the configuration of the cubin can be executed for the given params.
   bool isValidConfig(BatchedGemmConfig const& config, BatchedGemmData const& data) const;
@@ -480,10 +516,6 @@ class BatchedGemmInterface {
 
   // Returns the size padded to the alignment
   size_t getSizePaddedToAlignment(size_t size, size_t alignment) const;
-
- private:
-  // Number of the CTAs of the last launched kernel.
-  int32_t mNumCtas{0};
 };
 
 ////////////////////////////////////////////////////////////////////////////////////////////////////
@@ -644,29 +676,8 @@ int32_t BatchedGemmInterface::run(BatchedGemmConfig const& config, void* workspa
     }
   }
 
-  int32_t numCtaXy{0};
-  if (options.mIsStaticBatch) {
-    for (int32_t bi = 0; bi < options.mNumBatches; ++bi) {
-      numCtaXy += batchM ? gemm::divUp(options.mBatchedM[bi], options.mTileM)
-                         : gemm::divUp(options.mBatchedN[bi], options.mTileN);
-    }
-  }
-
-  int32_t maxNumCtasInBatchDim{numCtaXy};
-  // For normal BMM, mNumTokens == 0 and the number of CTAs is known to host.
-  // For MoE, mNumTokens != 0 and the number of CTAs is known only at runtime.
-  // We launch maximally possible number of CTAs and use ptrNumNonExitingCtas to determine
-  // the actual number of CTAs to run.
-  if ((options.mEnablesEarlyExit || options.mEnablesDelayedEarlyExit) && options.mNumTokens != 0) {
-    // Get maximum number of CTAs in batch dim.
-    maxNumCtasInBatchDim = batchedGemmData.mProblemDimensions.mMaxNumCtasInTokenDim;
-  }
-
-  auto const numCtaX = batchM ? maxNumCtasInBatchDim : gemm::divUp(options.mM, options.mTileM);
-  auto const numCtaY = batchM ? gemm::divUp(options.mN, options.mTileN) : maxNumCtasInBatchDim;
-  auto const numCtaZ = options.mNumSlicesForSplitK;
-  mNumCtas = numCtaX * numCtaY * numCtaZ;
-
+  auto [numCtaBatch, numCtaTile, numCtaInner] =
+      getGridDim(options, batchedGemmData.mProblemDimensions.mMaxNumCtasInTokenDim);
   auto kernelParams = KernelParamsSetup::setKernelParams(
       options, batchM, batchedGemmData.mInputBuffers.mPtrA, batchedGemmData.mInputBuffers.mPtrB,
       batchedGemmData.mOutputBuffers.mPtrC, batchedGemmData.mInputBuffers.mPtrSfA,
@@ -674,15 +685,16 @@ int32_t BatchedGemmInterface::run(BatchedGemmConfig const& config, void* workspa
       batchedGemmData.mInputBuffers.mPtrPerTokenSfB, batchedGemmData.mInputBuffers.mPtrBias,
       batchedGemmData.mOutputBuffers.mPtrSfC, batchedGemmData.mInputBuffers.mPtrScaleC,
       batchedGemmData.mInputBuffers.mPtrScaleGate, batchedGemmData.mInputBuffers.mPtrClampLimit,
-      batchedGemmData.mInputBuffers.mPtrSwiGluAlpha, batchedGemmData.mInputBuffers.mPtrSwiGluBeta,
-      batchedGemmData.mInputBuffers.mPtrRouteMap, dPtrRowMax, dPtrRowMaxBars,
-      batchedGemmData.mInputBuffers.mPtrNumNonExitingCtas,
+      batchedGemmData.mInputBuffers.mPtrGatedActAlpha,
+      batchedGemmData.mInputBuffers.mPtrGatedActBeta, batchedGemmData.mInputBuffers.mPtrRouteMap,
+      dPtrRowMax, dPtrRowMaxBars, batchedGemmData.mInputBuffers.mPtrNumNonExitingCtas,
       batchedGemmData.mInputBuffers.mPtrTotalNumPaddedTokens,
       batchedGemmData.mInputBuffers.mPtrCtaIdxXyToBatchIdx,
-      batchedGemmData.mInputBuffers.mPtrCtaIdxXyToMnLimit, maxNumCtasInBatchDim);
+      batchedGemmData.mInputBuffers.mPtrCtaIdxXyToMnLimit, numCtaBatch);
 
   // The size of the grid.
-  std::vector<int32_t> grid{numCtaX, numCtaY, numCtaZ};
+  std::vector<int32_t> grid = batchM ? std::vector<int32_t>{numCtaBatch, numCtaTile, numCtaInner}
+                                     : std::vector<int32_t>{numCtaTile, numCtaBatch, numCtaInner};
 
 #ifdef TLLM_GEN_EXPORT_INTERFACE
   CUmodule cuModule;
