@@ -46,7 +46,8 @@ def _run_correctness_worker(world_size, rank, dtype, hidden_dim, distributed_ini
         ]
         swizzled_layout_codes = [
             comm.QuantizationSFLayout.LINEAR,
-            comm.QuantizationSFLayout.SWIZZLED,
+            comm.QuantizationSFLayout.SWIZZLED_128x4,
+            comm.QuantizationSFLayout.SWIZZLED_8x4,
         ]
         launch_with_pdls = [True, False]
         use_oneshots = [True, False, None]
@@ -122,7 +123,7 @@ def _run_correctness_worker(world_size, rank, dtype, hidden_dim, distributed_ini
                                     )
                                     if (
                                         swizzled_layout_code
-                                        == comm.QuantizationSFLayout.SWIZZLED
+                                        == comm.QuantizationSFLayout.SWIZZLED_128x4
                                     ):
                                         # TODO(Yingyi): check this
                                         padded_message_size = (
